/******************************************************************************
* ADO.NETReadCS.cs - Read SAS data using the OleDbDataReader class            *
*                                                                             *
* To compile: csc /t:exe /r:System.dll,System.Data.dll ADO.NETReadCS.cs       *
*                                                                             *
* This program prints the contents of a dataset on an IOM Server to the       *
* console.                                                                    *
* It takes one command-line argument:                                         *
*     <libname>.<table name>             the SAS table to be read             *
* Example: ADO.NETReadCS mylib.mydata                                         *
******************************************************************************/

using System;
using System.IO;
using System.Data;
using System.Data.OleDb;

public class ADONETReadCS
{
    public static void Main()
    {
        // Get the command line arguments
        String[] args = Environment.GetCommandLineArgs();
        // The first element will be the name of the running program
        // so there should be two arguments in the list.
        if( args.Length < 2 )
        {
            Console.WriteLine( "Required arguments: <libname>.<tablename>." );
            Environment.Exit( -1 );
        }
        String itable = args[1];

        // Create a new OleDbConnection and tell it to use the IOM provider.
        // Other providers can be used by changing the ConnectionString.
        OleDbConnection cn = new OleDbConnection();
        cn.ConnectionString = "Provider=sas.IOMProvider.1; Data Source=_LOCAL_";

        try
        {
            // Open the connection and print the version number.
            cn.Open();
            Console.WriteLine( "SAS Server Version: " + cn.ServerVersion );

            // Create an OleDbCommand object and tell it
            // to open the specified table.
            OleDbCommand cmd  = cn.CreateCommand();
            cmd.CommandType = CommandType.TableDirect;
            cmd.CommandText = itable;
            
            // Execute the command and get an OleDbDataReader object.
            OleDbDataReader reader = cmd.ExecuteReader();
            
            // Get the Schema information so we can print out the column names.
            DataTable schema = reader.GetSchemaTable();
            Console.Write( "Columns: " );
            for( int i = 0; i < schema.Rows.Count; ++i )
            {
                if( i > 0 ){ Console.Write( ", " ); }
                Console.Write( schema.Rows[i]["ColumnName"] );
            }
            Console.WriteLine();

            // Print out the values in each row.
            Console.WriteLine( "Values: " );
            int nRows = 0;
            while( reader.Read() )
            {
                for( int j = 0; j < reader.FieldCount; ++j )
                {
                    if( j > 0 ){ Console.Write( ", " ); }
                    
                    // If the value is null then print "-missing-"
                    // otherwise convert the value to a string and print it.
                    if( reader.IsDBNull( j ) )
                    {
                        Console.Write( "-missing-" );
                    }
                    else
                    {
                        Console.Write( reader.GetValue( j ).ToString() );
                    }
                }
                Console.Write( "\n" );
                nRows += 1;
            }
            Console.WriteLine( "Read " + nRows + " rows." );
            reader.Close();
        }
        catch( Exception e )
        {
            Console.WriteLine( "Exception: " +  e.ToString() );
        }
        if ( ( cn != null) &&
             ( cn.State != ConnectionState.Closed ) )
        {
            cn.Close();
        }
        cn = null;
    }
}
