'*****************************************************************************'
' ADO.NETReadVB.vb - Read SAS data using the OleDbDataReader class            '
'                                                                             '
' To compile: vbc /t:exe /r:System.dll,System.Data.dll ADO.NETReadVB.vb       '
'                                                                             '
' This program prints the contents of a dataset on an IOM Server to the       '
' console.                                                                    '
' It takes one command-line argument:                                         '
'     <libname>.<table name>             the SAS table to be read             '
' Example: ADO.NETReadVB mylib.mydata                                         '
'*****************************************************************************'

Option Explicit On
Option Strict Off
Option Compare Text

Imports System
Imports System.IO
Imports System.Data
Imports System.Data.OleDb

Public Class ADONETReadVB

    Public Shared Sub Main()

        ' Get the command line arguments
        Dim args As String() = Environment.GetCommandLineArgs()
        ' The first element will be the name of the running program
        ' so there should be two arguments in the list.
        If args.Length < 2 Then
            Console.WriteLine( "Required arguments: <libname>.<tablename>." )
            Environment.Exit( -1 )
        End If
        Dim itable As String = args(1)

        ' Create a new OleDbConnection and tell it to use the IOM provider.
        ' Other providers can be used by changing the ConnectionString.
        Dim cn As OleDbConnection = New OleDbConnection()
        cn.ConnectionString = "Provider=sas.IOMProvider.1; Data Source=_LOCAL_"

        Try
            ' Open the connection and print the version number.
            cn.Open()
            Console.WriteLine( "SAS Server Version: " & cn.ServerVersion )

            ' Create an OleDbCommand object and tell it
            ' to open the specified table
            Dim cmd As OleDbCommand = cn.CreateCommand()
            cmd.CommandType = CommandType.TableDirect
            cmd.CommandText = itable

            ' Execute the command and get an OleDbDataReader object.
            Dim reader As OleDbDataReader = cmd.ExecuteReader()

            ' Get the Schema information so we can print out the column names.
            Dim schema As DataTable = reader.GetSchemaTable()
            Console.Write( "Columns: " )
            Dim i as Integer
            For i = 1 To schema.Rows.Count
                If i > 1 Then Console.Write( ", " )
                Console.Write( schema.Rows(i-1).Item("ColumnName") )
            Next
            Console.WriteLine()

            ' Print out the values in each row.
            Console.WriteLine( "Values: " )
            Dim nRows as Integer
            nRows = 0
            While reader.Read()
                For i = 1 To reader.FieldCount
                    If i > 1 Then Console.Write( ", " )

                    ' If the value is missing then print "-missing-"
                    ' otherwise convert the value to a string and print it.
                    If reader.IsDBNull(i-1) Then
                        Console.Write( "-missing-" )
                    Else
                        Console.Write( reader.GetValue(i-1).ToString() )
                    End If
                Next
                Console.WriteLine()
                nRows += 1
            End While
            Console.WriteLine( "Read " & nRows & " rows." )
            reader.Close()
        Catch e As Exception
            Console.WriteLine( "Exception: " & e.ToString() )
        End Try
        If ( Not cn Is Nothing ) AndAlso _
           ( cn.State <> ConnectionState.Closed ) Then _
           cn.Close()
        cn = Nothing

    End Sub

End Class
