/******************************************************************************
* ADOReadCS.cs - Read SAS data using ADO (classic) classes                    *
*                                                                             *
* To create ADODB.dll: tlibimp msado15.dll                                    *
* msado15.dll is typically located in C:\Program Files\Common Files\System\ado*
*                                                                             *
* To compile: csc /t:exe /r:System.dll,System.Data.dll,ADODB.dll ADOReadCS.cs *
*                                                                             *
* This program prints the contents of a dataset on an IOM Server to the       *
* console with all columns formatted with their default formats.              *
* It takes one command-line argument:                                         *
*     <libname>.<table name>             the SAS table to be read             *
* Example: ADOReadCS mylib.mydata                                             *
******************************************************************************/

using System;
using System.Data;
using System.Reflection;

public class ADOReadCS
{
    public static void Main()
    {
        // Get the command line arguments
        String[] args = Environment.GetCommandLineArgs();
        // The first element will be the name of the running program
        // so there should be two arguments in the list.
        if( args.Length < 2 )
        {
            Console.WriteLine( "Required arguments: <libname>.<tablename>." );
            Environment.Exit( -1 );
        }
        String itable = args[1];

        // Create a new ADO Connection object
        ADODB.Connection adoConnection = new ADODB.Connection();

        try
        {
            // Open the connection and print the version number.
            // Other providers can be used by changing the ConnectionString.
            adoConnection.Open("Provider=sas.IOMProvider.1; Data Source=_LOCAL_;", "", "", 0);
            Console.WriteLine( "SAS Server Version: " +
                               adoConnection.Properties["DBMS Version"].Value );
            
            // Open a Recordset on the specified dataset.
            ADODB.Recordset adoRecordset = new ADODB.Recordset();
            adoRecordset.ActiveConnection = adoConnection;
            
            // If you wanted to apply a format to this Recordset
            // you would do it here with something like
            // adoRecordset.Properties["SAS Formats"].Value = "_ALL_";
            
            // Perform the open
            adoRecordset.Open(itable, Missing.Value, ADODB.CursorTypeEnum.adOpenForwardOnly,
                ADODB.LockTypeEnum.adLockReadOnly, (int)ADODB.CommandTypeEnum.adCmdTableDirect );
            
            // Print the column names
            Console.Write( "Columns: " );
            for( int i = 0; i < adoRecordset.Fields.Count; ++i )
            {
                if( i > 0 ){ Console.Write( ", " ); }
                Console.Write( adoRecordset.Fields[i].Name );
            }
            Console.WriteLine();

            // Print out the values in each row.
            Console.WriteLine( "Values: " );
            int nRows = 0;
            while( !adoRecordset.EOF )
            {
                for( int j = 0; j < adoRecordset.Fields.Count; ++j )
                {
                    if( j > 0 ){ Console.Write( ", " ); }

                    // If a missing value was found print "-missing-"
                    // otherwise convert the value to a string and print it.
                    Type t = adoRecordset.Fields[j].Value.GetType();
                    if( Type.GetTypeCode( t ) == TypeCode.DBNull )
                    {
                        Console.Write( "-missing-" );
                    }
                    else
                    {
                        Console.Write( adoRecordset.Fields[j].Value.ToString() );
                    }
                }
                Console.WriteLine();
                adoRecordset.MoveNext();
                nRows += 1;
            }
            Console.WriteLine( "Read " + nRows + " rows." );
            adoRecordset.Close();
        }
        catch (Exception e)
        {
            Console.WriteLine( "Exception: " + e.ToString() );
        }
        if ( ( adoConnection != null ) &&
             ( adoConnection.State != (int)ADODB.ObjectStateEnum.adStateClosed ) )
        {
            adoConnection.Close();
        }
        adoConnection = null;
    }
}
