'*****************************************************************************'
' ADOReadVB.vb - Read SAS data using ADO (classic) classes                    '
'                                                                             '
' To create ADODB.dll: tlibimp msado15.dll                                    '
' msado15.dll is typically located in C:\Program Files\Common Files\System\ado'
'                                                                             '
' To compile: vbc /t:exe /r:System.dll,System.Data.dll,ADODB.dll ADOReadVB.vb '
'                                                                             '
' This program prints the contents of a dataset on an IOM Server to the       '
' console with all columns formatted with their default formats.              '
' It takes one command-line argument:                                         '
'     <libname>.<table name>             the SAS table to be read             '
' Example: ADOReadVB mylib.mydata                                             '
'*****************************************************************************'

Option Explicit On
Option Strict Off
Option Compare Text

Imports System
Imports System.Data
Imports System.Reflection

Public Class ADOReadVB

    Public Shared Sub Main()

        ' Get the command line arguments
        Dim args As String() = Environment.GetCommandLineArgs()
        ' The first element will be the name of the running program
        ' so there should be two arguments in the list.
        If args.Length < 2 Then
            Console.WriteLine( "Required arguments: <libname>.<tablename>." )
            Environment.Exit( -1 )
        End If
        Dim itable As String = args(1)

        ' Create a new ADO Connection object
        Dim adoConnection as ADODB.Connection = new ADODB.Connection()

        Try
            ' Open the connection and print the version number.
            ' Other providers can be used by changing the ConnectionString.
            adoConnection.Open("Provider=sas.IOMProvider.1; Data Source=_LOCAL_;", "", "", 0)
            Console.WriteLine( "SAS Server Version: " & _
                               adoConnection.Properties("DBMS Version").Value )

            ' Open a Recordset on the specified dataset.
            Dim adoRecordset as ADODB.Recordset = new ADODB.Recordset()
            adoRecordset.ActiveConnection = adoConnection

            ' If you wanted to apply a format to this Recordset
            ' you would do it here with something like
            ' adoRecordset.Properties("SAS Formats").Value = "_ALL_"

            ' Perform the open
            adoRecordset.Open(itable, Missing.Value, ADODB.CursorTypeEnum.adOpenForwardOnly, _
                ADODB.LockTypeEnum.adLockReadOnly, ADODB.CommandTypeEnum.adCmdTableDirect )
            
            ' Print the column names
            Console.Write( "Columns: " )
            Dim i as Integer
            For i = 1 To adoRecordset.Fields.Count
                If i > 1 Then Console.Write( ", " )
                Console.Write( adoRecordset.Fields(i-1).Name )
            Next
            Console.WriteLine()

            ' Print out the values in each row.
            Console.WriteLine( "Values: " )
            Dim nRows as Integer
            nRows = 0
            While Not adoRecordset.EOF
                For i = 1 To adoRecordset.Fields.Count
                    If i > 1 Then Console.Write( ", " )

                    ' If a missing value was found print "-missing-"
                    ' otherwise convert the value to a string and print it.
                    Dim t as Type
                    t = adoRecordset.Fields(i-1).Value.GetType()
                    If Type.GetTypeCode( t ) = TypeCode.DBNull Then
                        Console.Write( "-missing-" )
                    Else
                        Console.Write( adoRecordset.Fields(i-1).Value.ToString() )
                    End If
                Next
                Console.WriteLine()
                adoRecordset.MoveNext()
                nRows += 1
            End While
            Console.WriteLine( "Read " & nRows & " rows." )
            adoRecordset.Close()
        Catch e As Exception
            Console.WriteLine( "Exception: " & e.ToString() )
        End Try
        If ( Not adoConnection Is Nothing ) AndAlso _
           ( adoConnection.State <> ADODB.ObjectStateEnum.adStateClosed ) Then _
           adoConnection.Close()
        adoConnection = Nothing

    End Sub

End Class
